const std = @import("std");

pub fn Keys(comptime KeyType: type) type {
    return struct {
        const Self = @This();

        pressed: std.AutoHashMap(KeyType, bool),

        pub fn init(allocator: std.mem.Allocator) Self {
            return Self{
                .pressed = std.AutoHashMap(KeyType, bool).init(allocator),
            };
        }

        pub fn deinit(self: *Self) void {
            self.pressed.deinit();
        }

        pub fn isKeyPressed(self: Self, key: KeyType) bool {
            if (self.pressed.get(key)) |pressed| {
                return pressed;
            }
            return false;
        }

        pub fn press(self: *Self, key: KeyType) !void {
            try self.pressed.put(key, true);
        }

        pub fn release(self: *Self, key: KeyType) !void {
            try self.pressed.put(key, false);
        }
    };
}
