const std = @import("std");
const Allocator = std.mem.Allocator;

pub const OpaquePtr = struct {
    allocator: Allocator,
    object: usize = undefined,
    virtualTable: VirtualTable = undefined,

    const VirtualTable = struct {
        destroy: *const fn (OpaquePtr) void,
    };

    pub fn Object(comptime Type: type) type {
        return struct {
            ptr: *Type,
            opaquePtr: OpaquePtr,
        };
    }

    pub fn create(comptime Type: type, allocator: Allocator) !Object(Type) {
        const ptr = try allocator.create(Type);
        const opaq = OpaquePtr{
            .allocator = allocator,
            .object = @ptrToInt(ptr),
            .virtualTable = .{
                .destroy = struct {
                    fn destroy(self: OpaquePtr) void {
                        const object = @intToPtr(*Type, self.object);
                        self.allocator.destroy(object);
                    }
                }.destroy,
            },
        };
        return Object(Type){
            .ptr = ptr,
            .opaquePtr = opaq,
        };
    }

    pub fn destroy(self: OpaquePtr) void {
        self.virtualTable.destroy(self);
    }
};
