const Point = @import("point.zig").Point;
const RectangleF = @import("rectangle_f.zig").RectangleF;

pub const Rectangle = struct {
    min: Point,
    max: Point,

    pub fn init(min: Point, max: Point) Rectangle {
        if (min.x > max.x) unreachable;
        if (min.y > max.y) unreachable;

        return Rectangle{ .min = min, .max = max };
    }

    pub fn initAbsolute(x1: i64, y1: i64, x2: i64, y2: i64) Rectangle {
        if (x1 < x2) {
            if (y1 < y2) {
                return Rectangle{
                    .min = Point{ .x = x1, .y = y1 },
                    .max = Point{ .x = x2, .y = y2 },
                };
            }
            return Rectangle{
                .min = Point{ .x = x1, .y = y2 },
                .max = Point{ .x = x2, .y = y1 },
            };
        }
        if (y1 < y2) {
            return Rectangle{
                .min = Point{ .x = x2, .y = y1 },
                .max = Point{ .x = x1, .y = y2 },
            };
        }
        return Rectangle{
            .min = Point{ .x = x2, .y = y2 },
            .max = Point{ .x = x1, .y = y1 },
        };
    }

    pub fn initRelative(x: i64, y: i64, w: i64, h: i64) Rectangle {
        if (w < 0) {
            if (h < 0) {
                return Rectangle{
                    .min = Point{ .x = x + w, .y = y + h },
                    .max = Point{ .x = x, .y = y },
                };
            }
            return Rectangle{
                .min = Point{ .x = x + w, .y = y },
                .max = Point{ .x = x, .y = y + h },
            };
        }
        if (h < 0) {
            return Rectangle{
                .min = Point{ .x = x, .y = y + h },
                .max = Point{ .x = x + w, .y = y },
            };
        }
        return Rectangle{
            .min = Point{ .x = x, .y = y },
            .max = Point{ .x = x + w, .y = y + h },
        };
    }

    pub fn center(self: Rectangle) Point {
        return Point.init(@divTrunc(self.min.x + self.max.x, 2), @divTrunc(self.min.y + self.max.y, 2));
    }

    pub fn float(self: Rectangle) RectangleF {
        return RectangleF{ .min = self.min.float(), .max = self.max.float() };
    }

    pub fn height(self: Rectangle) i64 {
        return self.max.y - self.min.y;
    }

    pub fn width(self: Rectangle) i64 {
        return self.max.x - self.min.x;
    }
};
