const PointF = @import("point_f.zig").PointF;
const Rectangle = @import("rectangle.zig").Rectangle;

pub const RectangleF = struct {
    min: PointF,
    max: PointF,

    pub fn init(min: PointF, max: PointF) RectangleF {
        if (min.x > max.x) unreachable;
        if (min.y > max.y) unreachable;

        return RectangleF{ .min = min, .max = max };
    }

    pub fn initAbsolute(x1: f32, y1: f32, x2: f32, y2: f32) RectangleF {
        if (x1 < x2) {
            if (y1 < y2) {
                return RectangleF{
                    .min = PointF{ .x = x1, .y = y1 },
                    .max = PointF{ .x = x2, .y = y2 },
                };
            }
            return RectangleF{
                .min = PointF{ .x = x1, .y = y2 },
                .max = PointF{ .x = x2, .y = y1 },
            };
        }
        if (y1 < y2) {
            return RectangleF{
                .min = PointF{ .x = x2, .y = y1 },
                .max = PointF{ .x = x1, .y = y2 },
            };
        }
        return RectangleF{
            .min = PointF{ .x = x2, .y = y2 },
            .max = PointF{ .x = x1, .y = y1 },
        };
    }

    pub fn initRelative(x: f32, y: f32, w: f32, h: f32) RectangleF {
        if (w < 0) {
            if (h < 0) {
                return RectangleF{
                    .min = PointF{ .x = x + w, .y = y + h },
                    .max = PointF{ .x = x, .y = y },
                };
            }
            return RectangleF{
                .min = PointF{ .x = x + w, .y = y },
                .max = PointF{ .x = x, .y = y + h },
            };
        }
        if (h < 0) {
            return RectangleF{
                .min = PointF{ .x = x, .y = y + h },
                .max = PointF{ .x = x + w, .y = y },
            };
        }
        return RectangleF{
            .min = PointF{ .x = x, .y = y },
            .max = PointF{ .x = x + w, .y = y + h },
        };
    }

    pub fn center(self: RectangleF) PointF {
        return self.min.add(self.max).multiply(0.5);
    }

    pub fn height(self: RectangleF) f32 {
        return self.max.y - self.min.y;
    }

    pub fn integer(self: RectangleF) Rectangle {
        return Rectangle{ .min = self.min.integer(), .max = self.max.integer() };
    }

    pub fn width(self: RectangleF) f32 {
        return self.max.x - self.min.x;
    }
};
