const std = @import("std");
const PointF = @import("point_f.zig").PointF;
const RectangleF = @import("rectangle_f.zig").RectangleF;

pub const Viewport = struct {
    unscaledWidth: i32,
    unscaledHeight: i32,

    actualWidth: i32,
    actualHeight: i32,

    scaledWidth: i32,
    scaledHeight: i32,
    bounds: RectangleF,
    scale: f32,

    pub fn init(width: i32, height: i32) Viewport {
        return Viewport{
            .unscaledWidth = width,
            .unscaledHeight = height,
            .actualWidth = width,
            .actualHeight = height,
            .scaledWidth = width,
            .scaledHeight = height,
            .bounds = RectangleF.initRelative(0, 0, @intToFloat(f32, width), @intToFloat(f32, height)),
            .scale = 1.0,
        };
    }

    pub fn center(self: Viewport) PointF {
        return self.bounds.center();
    }

    pub fn scaledInteger(self: Viewport, i: i32) i32 {
        return @floatToInt(i32, std.math.round(@intToFloat(f32, i) * self.scale));
    }

    pub fn viewToScreen(self: Viewport, x: f32, y: f32) PointF {
        const screenX = self.bounds.min.x + x * self.bounds.width();
        const screenY = self.bounds.min.y + y * self.bounds.height();
        return PointF.init(screenX, screenY);
    }

    pub fn viewToScreenP(self: Viewport, position: PointF) PointF {
        return self.viewToScreen(position.x, position.y);
    }

    pub fn update(self: *Viewport, width: i32, height: i32) void {
        self.actualWidth = width;
        self.actualHeight = height;

        const horizontalRatio = @intToFloat(f32, width) / @intToFloat(f32, self.unscaledWidth);
        const verticalRatio = @intToFloat(f32, height) / @intToFloat(f32, self.unscaledHeight);

        if (horizontalRatio < verticalRatio) {
            self.scaledWidth = width;
            self.scaledHeight = @floatToInt(i32, horizontalRatio * @intToFloat(f32, self.unscaledHeight));
            const top = @divFloor(height - self.scaledHeight, 2);
            self.bounds = RectangleF.initRelative(0, @intToFloat(f32, top), @intToFloat(f32, width), @intToFloat(f32, self.scaledHeight));
            self.scale = horizontalRatio;
        } else {
            self.scaledWidth = @floatToInt(i32, verticalRatio * @intToFloat(f32, self.unscaledWidth));
            self.scaledHeight = height;
            const left = @divFloor(width - self.scaledWidth, 2);
            self.bounds = RectangleF.initRelative(@intToFloat(f32, left), 0, @intToFloat(f32, self.scaledWidth), @intToFloat(f32, height));
            self.scale = verticalRatio;
        }
    }
};
