const allegro = @import("allegro");
const engine = @import("../engine.zig");

pub const Animation = struct {
    sprite: engine.Sprite,
    current: usize,
    begin: usize,
    end: usize,
    interval: f32,
    lastUpdate: f32,

    pub fn init(sprite: engine.Sprite, interval: f32) Animation {
        return Animation.initPartialLoop(sprite, interval, 0, sprite.frames.len);
    }

    pub fn initPartialLoop(sprite: engine.Sprite, interval: f32, begin: usize, end: usize) Animation {
        return Animation{
            .sprite = sprite,
            .current = begin,
            .begin = begin,
            .end = end,
            .interval = interval,
            .lastUpdate = 0,
        };
    }

    pub fn currentFrame(self: *Animation) allegro.Bitmap {
        return self.sprite.frames[self.current];
    }

    pub fn currentFrameOffset(self: *Animation, offset: usize) allegro.Bitmap {
        return self.sprite.frames[self.currentOffset(offset)];
    }

    pub fn currentOffset(self: Animation, offset: usize) usize {
        const n = self.end - self.begin;
        return self.begin + (self.current + offset - self.begin) % n;
    }

    pub fn reset(self: *Animation) void {
        self.current = self.begin;
    }

    pub fn tick(self: *Animation, t: f32) void {
        const delta = t - self.lastUpdate;

        const skip = @floatToInt(usize, @divFloor(delta, self.interval));
        self.lastUpdate = t - @rem(delta, self.interval);

        self.current = self.currentOffset(skip);
    }
};
